import { DesktopNotificationPermission } from './notification-permission';
import { INotificationOptions } from './notification-options';
/** Initializes the desktop-notifications system. */
export declare const initializeNotifications: (opts: INotificationOptions) => void;
/** Terminates the desktop-notifications system. */
export declare const terminateNotifications: () => void;
/** Gets the current state of the notifications permission. */
export declare const getNotificationsPermission: () => Promise<DesktopNotificationPermission>;
/** Requests the user to grant permission to display notifications. */
export declare const requestNotificationsPermission: () => Promise<boolean>;
/**
 * Displays a system notification.
 * @param title Title of the notification
 * @param body Body of the notification
 * @param userInfo (Optional) An object with any information that needs to be
 * passed to the notification callback when the user clicks on the notification.
 * @returns The ID of the notification displayed. This ID can be used to close
 * the notification.
 */
export declare const showNotification: (title: string, body: string, userInfo?: Record<string, any>) => Promise<string | null>;
/** Closes the notification with the given ID. */
export declare const closeNotification: (id: string) => void;
